package edu.cmu.ece.drawe3.render;


import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.app.Activity;
import android.opengl.GLU;
import edu.cmu.ece.drawe3.comm.PointReceiver;
import edu.cmu.ece.drawe3.ui.StatusReport;

public class OpenGLRenderer implements D3Renderer {
	
	public Points points;
	
	float[] modeltrans;
	float[] projection;
	private boolean resetProj = false;
	
	PointReceiver recv;
	
	public OpenGLRenderer(Activity activity, String host, StatusReport.Field sp)
	{
		this.recv = new PointReceiver(this, host, activity, sp);
		this.points = new Points();
		
		this.recv.start();
	}
	
	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * android.opengl.GLSurfaceView.Renderer#onSurfaceCreated(javax.
         * microedition.khronos.opengles.GL10, javax.microedition.khronos.
         * egl.EGLConfig)
	 */
	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		
	    gl.glDisable(GL10.GL_DITHER);
	    gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT,
	            GL10.GL_FASTEST);

	    gl.glEnable(GL10.GL_CULL_FACE);
	     
		// Set the background color to black ( rgba ).
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // OpenGL docs.
		// Enable Smooth Shading, default not really needed.
		gl.glShadeModel(GL10.GL_SMOOTH);// OpenGL docs.
		// Depth buffer setup.
		gl.glClearDepthf(1.0f);// OpenGL docs.
		// Enables depth testing.
		gl.glEnable(GL10.GL_DEPTH_TEST);// OpenGL docs.
		// The type of depth testing to do.
		gl.glDepthFunc(GL10.GL_LEQUAL);// OpenGL docs.
		// Really nice perspective calculations.
		gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, // OpenGL docs.
                          GL10.GL_NICEST);
	}
	
	
	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * android.opengl.GLSurfaceView.Renderer#onDrawFrame(javax.
         * microedition.khronos.opengles.GL10)
	 */
	public void onDrawFrame(GL10 gl) {
		
		// Clears the screen and depth buffer.
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | // OpenGL docs.
                           GL10.GL_DEPTH_BUFFER_BIT);
		
		if (resetProj)
        {
        	gl.glMatrixMode(GL10.GL_PROJECTION);
            if (this.projection != null)
            	gl.glLoadMatrixf(projection, 0);
            else
            	gl.glLoadIdentity();
            this.resetProj = false;
        }
		
		gl.glMatrixMode(GL10.GL_MODELVIEW);
        if (this.modeltrans != null)
        	gl.glLoadMatrixf(this.modeltrans, 0);
        else
        	gl.glLoadIdentity();
		
		
		
		gl.glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
		//gl.glTranslatef(0, 0, -2.5f);
		points.draw(gl);
		/*
	    Square square = new Square();
	    Cube cube = new Cube();
    	
	  	
	    square.draw(gl);
    	*/
    	
		
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * android.opengl.GLSurfaceView.Renderer#onSurfaceChanged(javax.
         * microedition.khronos.opengles.GL10, int, int)
	 */
	public void onSurfaceChanged(GL10 gl, int width, int height) {
		// Sets the current view port to the new size.
		gl.glViewport(0, 0, width, height);// OpenGL docs.
		// Select the projection matrix
		gl.glMatrixMode(GL10.GL_PROJECTION);// OpenGL docs.
		// Reset the projection matrix
		
		if (this.projection != null)
       	 	gl.glLoadMatrixf(projection, 0);
		else
		{
			gl.glLoadIdentity();// OpenGL docs.
			GLU.gluPerspective(gl, 45.0f,
                                   (float) width / (float) height,
                                   0.1f, 100.0f);
		}
		/*gl.glOrthof(0.0f, 10.0f, 0.0f, 10.0f, -10.0f, 20.0f);*/
		// Select the modelview matrix
		gl.glMatrixMode(GL10.GL_MODELVIEW);// OpenGL docs.
        if (this.modeltrans != null)
        	gl.glLoadMatrixf(this.modeltrans, 0);
        else
        	gl.glLoadIdentity();
	}

	public void setProjection(float[] proj) {
		this.projection = proj;
		this.resetProj = true;
	}

	public void setModelView(float[] _trans) {
		this.modeltrans = _trans;
	}
}

