package edu.cmu.ece.drawe3.render;

import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;
import java.util.Scanner;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLU;


	public class StaticRenderer implements D3Renderer{
		
		FloatBuffer vertexBuffer;
		ShortBuffer indexBuffer;
		int index_size = 0;
		int num_points = 0;
		
		float[] modeltrans;
		float[] projection;
		private boolean resetProj = false;
		
		/*
		 * (non-Javadoc)
		 *
		 * @see
		 * android.opengl.GLSurfaceView.Renderer#onSurfaceCreated(javax.
	         * microedition.khronos.opengles.GL10, javax.microedition.khronos.
	         * egl.EGLConfig)
		 */
		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			
		    gl.glDisable(GL10.GL_DITHER);
		    gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT,
		            GL10.GL_FASTEST);

		    gl.glEnable(GL10.GL_CULL_FACE);
		     
			// Set the background color to black ( rgba ).
			gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // OpenGL docs.
			// Enable Smooth Shading, default not really needed.
			gl.glShadeModel(GL10.GL_SMOOTH);// OpenGL docs.
			// Depth buffer setup.
			gl.glClearDepthf(1.0f);// OpenGL docs.
			// Enables depth testing.
			gl.glEnable(GL10.GL_DEPTH_TEST);// OpenGL docs.
			// The type of depth testing to do.
			gl.glDepthFunc(GL10.GL_LEQUAL);// OpenGL docs.
			// Really nice perspective calculations.
			gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, // OpenGL docs.
	                          GL10.GL_NICEST);
		}


		public void load_points(InputStream in){
			Scanner scanner = new Scanner(in);
			num_points = scanner.nextInt();
			index_size = scanner.nextInt();
			
			float[] points = new float[num_points*3];
			short[] indices = new short[index_size*3];
			System.out.println("POINTS:" + num_points + "," + index_size);
			
			int p = 0;
			int i = 0;
			
			//flush
			scanner.nextLine(); 
			
			while(scanner.hasNextLine()){
				
				Scanner s = new Scanner(scanner.nextLine());
				String temp = s.next();
				
				//face
				if(temp.compareTo("f") == 0){
					int x = 0;
					while(s.hasNext()){
						if(x==3) break;
						indices[i++] = s.nextShort();
						x++;
					}
				}
				//vertex
				else{
					int x = 0;
					while(s.hasNext()){
						if(x==3) break;
						points[p++] = 300*s.nextFloat(); 
						x++;
					}
				}
				
			}
			
			// a float is 4 bytes, therefore we multiply the number if
			// vertices with 4.
			ByteBuffer vbb = ByteBuffer.allocateDirect(points.length * 4);
			vbb.order(ByteOrder.nativeOrder());
			vertexBuffer = vbb.asFloatBuffer();
			vertexBuffer.put(points);
			vertexBuffer.position(0);

			// short is 2 bytes, therefore we multiply the number if
			// vertices with 2.
			ByteBuffer ibb = ByteBuffer.allocateDirect(indices.length * 2);
			ibb.order(ByteOrder.nativeOrder());
			indexBuffer = ibb.asShortBuffer();  
			indexBuffer.put(indices);
			indexBuffer.position(0);
		}
		/*
		 * (non-Javadoc)
		 *
		 * @see
		 * android.opengl.GLSurfaceView.Renderer#onDrawFrame(javax.
	         * microedition.khronos.opengles.GL10)
		 */
		public void onDrawFrame(GL10 gl) {
			
	    	if(num_points==0) return; 
	    	
	    	
			// Clears the screen and depth buffer.
			gl.glClear(GL10.GL_COLOR_BUFFER_BIT | // OpenGL docs.
	                           GL10.GL_DEPTH_BUFFER_BIT);
			
			if (resetProj)
	        {
	        	gl.glMatrixMode(GL10.GL_PROJECTION);
	            if (this.projection != null)
	            	gl.glLoadMatrixf(projection, 0);
	            else
	            	gl.glLoadIdentity();
	            this.resetProj = false;
	        }
			
			gl.glMatrixMode(GL10.GL_MODELVIEW);
	        if (this.modeltrans != null)
	        	gl.glLoadMatrixf(this.modeltrans, 0);
	        else
	        	gl.glLoadIdentity();
			
			
			
			gl.glColor4f(0.0f, 1.0f, 0.0f, 0.5f);
			
			// Counter-clockwise winding.
			gl.glFrontFace(GL10.GL_CCW); // OpenGL docs
			// Enable face culling.
			gl.glEnable(GL10.GL_CULL_FACE); // OpenGL docs
			// What faces to remove with the face culling.
			gl.glCullFace(GL10.GL_BACK); // OpenGL docs

			// Enabled the vertices buffer for writing and to be used during
			// rendering.
			gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);// OpenGL docs.
			
			  gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
			  
			// Specifies the location and data format of an array of vertex
			// coordinates to use when rendering.
			
	        gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertexBuffer);
	        gl.glDrawElements(GL10.GL_TRIANGLES, index_size, GL10.GL_UNSIGNED_SHORT,indexBuffer);
	        

			// Disable the vertices buffer.
			gl.glDisableClientState(GL10.GL_VERTEX_ARRAY); // OpenGL docs
			gl.glDisableClientState(GL10.GL_COLOR_ARRAY);
			// Disable face culling.
			gl.glDisable(GL10.GL_CULL_FACE); // OpenGL docs
			
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see
		 * android.opengl.GLSurfaceView.Renderer#onSurfaceChanged(javax.
	         * microedition.khronos.opengles.GL10, int, int)
		 */
		public void onSurfaceChanged(GL10 gl, int width, int height) {
			// Sets the current view port to the new size.
			gl.glViewport(0, 0, width, height);// OpenGL docs.
			// Select the projection matrix
			gl.glMatrixMode(GL10.GL_PROJECTION);// OpenGL docs.
			// Reset the projection matrix
			
			if (this.projection != null)
	       	 	gl.glLoadMatrixf(projection, 0);
			else
			{
				gl.glLoadIdentity();// OpenGL docs.
				GLU.gluPerspective(gl, 45.0f,
	                                   (float) width / (float) height,
	                                   0.1f, 100.0f);
			}
			/*gl.glOrthof(0.0f, 10.0f, 0.0f, 10.0f, -10.0f, 20.0f);*/
			// Select the modelview matrix
			gl.glMatrixMode(GL10.GL_MODELVIEW);// OpenGL docs.
	        if (this.modeltrans != null)
	        	gl.glLoadMatrixf(this.modeltrans, 0);
	        else
	        	gl.glLoadIdentity();
		}

		public void setProjection(float[] proj) {
			this.projection = proj;
			this.resetProj = true;
		}

		public void setModelView(float[] _trans) {
			this.modeltrans = _trans;
		}
}
