package edu.cmu.ece.drawe3.tracking;

import java.util.Arrays;
import java.util.concurrent.Semaphore;
import java.util.concurrent.locks.Lock;

import com.studierstube.ARTKP.*;

import edu.cmu.ece.drawe3.tracking.nyartk.CameraCalibration;
import edu.cmu.ece.drawe3.tracking.nyartk.SingleNyIdMarker;
import edu.cmu.ece.drawe3.ui.StatusReport;
import edu.cmu.ece.drawe3.ui.Tracking;

import jp.nyatla.nyartoolkit.NyARException;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.Size;

public class TrackingHandler implements PreviewCallback
{
	//TODO: load size from MarkerCfg
	public static final int MARKER_SIZE = 100;
	//private Map<Integer, MarkerCfg> markerCfg;
	
	private StatusReport.Field sp;
	
	public TrackingHandler(Tracking tracking, String host, StatusReport.Field sp)
	{
		this.sp = sp;
		
		this.mTracking = tracking;
		
		CameraSource cs = new CameraSource(tracking);
		cs.setPreviewCallback(this);
		
		tracking.setProjection(CameraCalibration.getProjection(cs.getSize()));
		System.out.println(Arrays.toString(CameraCalibration.getProjection(cs.getSize())));
		
		//mInertialTracker = new InertialTracker(this, (SensorManager)getSystemService(Context.SENSOR_SERVICE), sp2);
		
		/*try {
			this.markerCfg = MarkerCfg.load(new URL("http", host, 80, "/markers.cfg"));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}*/
		
		fp.start();
	}
	
	private boolean skip = true;
	public void onPreviewFrame(byte[] data, Camera camera) {
		if (this.lock.tryAcquire())
		{
			this.frame_data = data;
			this.camera = camera;
			this.sem.release();
			this.lock.release();
		}
		Thread.yield();
		
		/*skip = !skip;
		if (skip)
			return;
		processFrame();*/
	}
	
	private FrameProcessor fp = new FrameProcessor();
	private Semaphore sem = new Semaphore(0);
	private Semaphore lock = new Semaphore(1);
	private byte[] frame_data;
	private Camera camera;
	
	public void processFrame()
	{
		if (mDetector == null)
		{
			sp.setVal("Initializing...");
			
			Camera.Size size = camera.getParameters().getPreviewSize();
			
			try {
				mDetector = new SingleNyIdMarker(CameraCalibration.getCalibration(camera.new Size(size.width, size.height)), MARKER_SIZE);
			} catch (NyARException e) {
				e.printStackTrace();
			}
		}
		
		if (mDetector.updateBuffer(frame_data) >= 0)
			sp.setVal("Track");
		else
			sp.setVal("Lost");
		
		double[] it = mDetector.getModelView();
		if (it != null)
		{
			float[] ot = new float[it.length];
			for(int i = 0; i < it.length; i++)
				ot[i] = (float) it[i];
			ot[13] += -15.0f;
			mTracking.setTransform(ot);
		}
		else
		{
			mTracking.setTransform(null);
		}
		
		/*Camera.Size size = camera.getParameters().getPreviewSize();
		
		int id = ARTKP.process(data, size.width, size.height);
		System.out.println("id: " + id);
		
		float[] xf = ARTKP.getModelViewMatrix();
		//float[] mxf = markerCfg.get(id).getPosition(xf);
		mTracking.setTransform(xf);
		mTracking.setProjection(ARTKP.getProjectionMatrix());*/
	}
	
	class FrameProcessor extends Thread
	{
		public FrameProcessor()
		{
			setDaemon(true);
		}
		
		public void run() {
			while(!Thread.interrupted())
			{
				try {
					sem.acquire();
					lock.acquire();
				} catch (InterruptedException e1) {
					continue;
				}
				sem.drainPermits();
				
				processFrame();
				
				lock.release();
			}
		}
	}
	
	private Tracking mTracking;
	private SingleNyIdMarker mDetector = null;
}