/* kinect.c
 * Author: Michael Ralph
 * Created: Wednesday, March 30, 2011
 * Last updated: Thursday, March 31, 2001
 */

#include <stdlib.h>
#include <stdio.h>
#include <libfreenect/libfreenect.h>
#include "Common.h"
#include <unistd.h>

#include <opencv/cv.h>
#include <opencv/cxcore.h>
#include <opencv/highgui.h>
#include <vector>

using namespace cv;
using namespace std;

/* includes for semaphores */
#include <fcntl.h>
#include <sys/stat.h>
#include <semaphore.h>

#include <GL/glut.h>

#include "trackball.h"

#include "arduino-kinect.h"

/*#define THRESH 100*/
#define THRESH 229
#define PORT 1096
#define PACKET_SIZE 50
#define R 0
#define G 1
#define B 2
#define X 0
#define Y 1
#define Z 2
#define NUM_ITERS 500
#define SKIP_ITERS 3

#define LINE_RED   1.0
#define LINE_GREEN 0.0
#define LINE_BLUE  0.0

typedef struct{
	/* number of sets of coordinates */
	int size;
	/* controller ID of remote sending (color?) */
	int controllerId;
	/* order being sent */
	int seq;
	/* array of coords */
	int arr[PACKET_SIZE*3];
} coord_packet;

struct Vertex
{
    Vertex(float _x, float _y, float _z, float _r, float _g, float _b) :
        x(_x), y(_y), z(_z), r(_r), g(_g), b(_b) {}
    
	float x, y, z;
	float r, g, b;
};

vector<Vertex> vertices;

static int ITERS = 0;
static coord_packet cp;
static Mat depth_filtered, depth_diff;
static sem_t semmy;
freenect_context* ctx;
freenect_device* dev;
int serv_fd;
static bool detected = false;

int sfd;

static Mat base_depth;
#define DEPTH_INIT_COUNT  20
static int depth_init = DEPTH_INIT_COUNT;

double d_floor = 2047;

const float mproj[] = { 2.1915433, 0.0, 0.0, 0.0,
                        0.0, 3.0273633, 0.0, 0.0,
                        0.010454323, 0.0069055376, -1.002002, -1.0,
                        0.0, 0.0, -20.02002, 0.0 };
const float mmv[] = { 0.08499895, -0.7582636, 0.6463834, 0.0,
                      0.99502915, 0.030815512, -0.09469636, 0.0,
                      0.051886167, 0.6512194, 0.7571137, 0.0,
                      -8.020124, -32.807003, -726.1499, 1.0 };

int isDrawing = false;


void display(void)
{
	glClearColor(1.0, 1.0, 1.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	glLineWidth(4.0);
	
	glPushMatrix();
    tbMatrix();
    

	glBegin(GL_LINE_STRIP);
		for(vector<Vertex>::iterator v = vertices.begin();
		    v != vertices.end(); v++)
		{
			glColor3f(v->r, v->g, v->b);
			glVertex3f(v->x, v->y, v->z);
		}
	glEnd();
	
	glPopMatrix();

	glutSwapBuffers();
}

void reshape(int w, int h)
{
    tbReshape(w, h);
	glViewport(0, 0, w, h);
}

/*void initNyARTK
{
	NyARParam ap;

	ap.setEndian(TNyAREndian_LITTLE);
	ap.loadARParamFromFile(CAMERA_FILE);
	ap.changeScreenSize(320, 240);


	unsigned char* buf;
	NyARRgbRaster_BGRA* ra;
	NyARSingleDetectMarker* ar;


	NyARCode code(16, 16);
	code.loadARPattFromFile(CODE_FILE);
	buf=load_file_image(DATA_FILE);
	ra = new NyARRgbRaster_BGRA(320, 240,false);
	ra->wrapBuffer(buf);

	ar = new NyARSingleDetectMarker(&ap,&code, 80.0,ra->getBufferType());
	ar->setContinueMode(false);

	NyARTransMatResult result_mat;

	ar->detectMarkerLite(*ra, 100);
	ar->getTransmationMatrix(result_mat);
	printf("%s",
		"performance test of NyARToolkit.\n"
		"This test measures processing time of marker detection and transform matrix calculation 1000 times.\n");
	printf("Marker confidence\n cf=%f\n",ar->getConfidence());
	printf("Transform Matrix\n");
	printf(
		"% 4.8f,% 4.8f,% 4.8f,% 4.8f\n"
		"% 4.8f,% 4.8f,% 4.8f,% 4.8f\n"
		"% 4.8f,% 4.8f,% 4.8f,% 4.8f\n",
		result_mat.m00,result_mat.m01,result_mat.m02,result_mat.m03,
		result_mat.m10,result_mat.m11,result_mat.m12,result_mat.m13,
		result_mat.m20,result_mat.m21,result_mat.m22,result_mat.m23);
		
	//DWORD st=GetTickCount();


	for (int i = 0; i < 1000; i++) {

		ar->detectMarkerLite(*ra, 100);
		ar->getTransmationMatrix(result_mat);
	}
	//printf("done.\ntotal=%u[ms]\n",GetTickCount()-st);
	printf("done.\ntotal=--[ms]\n");
	delete ra;
	delete ar;
	delete buf;
}*/


/* intializes the Kinect and server communication
 * returns 0 on success, < 0 on failure
 */
int knct_init(){
	printf("Server Starting\n");
	/* Find way to start the (java?) server */
	printf("Connecting to server\n");
	/*connect_to_serv(); */
	
	printf("Init'ing Kinect\n");
	if(freenect_init(&ctx, NULL) < 0){
		printf("Unable to init Kinect context\n");
		return -1;
	}
	if(freenect_num_devices(ctx) != 1){
		printf("I can't find my Kinect . . . :(\n");
		return -2;
	}
	
	printf("Opening Kinect Device\n");
	if(freenect_open_device(ctx, &dev, 0) < 0){
		printf("Unable to open Kinect Device\n");
		return -3;
	}
	printf("Opened Kinect Device\n");
	freenect_set_led(dev, LED_YELLOW);
	return 0;
}

void knct_cleanup(){
    printf("Stopping video\n");
	freenect_stop_video(dev);    
    
	printf("Disconnecting\n");
	/* disconnect from server and tell server to shutdown? */
	
	printf("Shutdown Kinect\n");
	freenect_set_led(dev, LED_BLINK_GREEN);
	if(freenect_close_device(dev) < 0){
		printf("Failed to close Kinect device\n");
	}
	if(freenect_shutdown(ctx) != 0){
		printf("Failed to shutdown the Kinect context\n");
	}
}

void vid_callback(freenect_device* dev, void* video, uint32_t timestamp){
	/* total number of packets that have been sent */
	
	ITERS++;
	if(ITERS < 5)
		return;
	
	if(ITERS%SKIP_ITERS!=0)
		return;
	
	if (!depth_filtered.rows) return;
	
	Mat frame(FREENECT_FRAME_H, FREENECT_FRAME_W, CV_8UC3, video);
	Mat gray, hls;
	vector<Mat> channels, channels2;
	cvtColor(frame, hls, CV_RGB2HLS);
	cvtColor(frame, gray, CV_RGB2GRAY);
	split(hls, channels);
	split(frame, channels2);
	
	/*sem_wait(&semmy);
	gray32 *= depth_filtered;
	sem_post(&semmy);*/
	
	Mat threshR, threshG, threshB, thresh, colored;
	//double maxVal = 255;
	//minMaxLoc(gray, NULL, &maxVal);
	/*threshold(channels2[0], threshR, 150, 255, THRESH_BINARY_INV);
	threshold(channels[2], threshG, 80, 255, THRESH_BINARY);
	threshold(channels[0], threshB, 35, 255, THRESH_BINARY_INV);
	
	vector<Mat> threshs;
	threshs.push_back(threshB);
	threshs.push_back(threshG);
	threshs.push_back(threshR);
	merge(threshs, colored);
	
	bitwise_and(threshR, threshB, thresh);
	bitwise_and(thresh, threshG, thresh);*/
	
	//printf("%d %d %d %d\n",depth_filtered.size().width,depth_filtered.size().height,depth_diff.size().width,depth_diff.size().height);
	threshold(depth_diff, threshR, 10, 255, THRESH_BINARY);
	threshold(depth_filtered, threshG, 100, 255, THRESH_BINARY_INV);
	multiply(threshR, threshG, thresh);
	
	vector< vector<Point> > contours;
	vector<Vec4i> hierarchy;
	contours.clear();
	findContours(thresh, contours, CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE);
        
    // iterate through all the top-level contours,
    // draw each connected component with its own random color
    int idx = 0;
	
	int num_elm = 0;
	
	if (contours.size() > 0)
	{
	    int i = 0;
        for(vector< vector<Point> >::iterator iter = contours.begin();
            iter != contours.end(); iter++, i++) {
	        
	        Point2f center;
	        float radius;
	        
	        Mat contour(contours[idx]);
	        
	        minEnclosingCircle(contour, center, radius);
	        
	        if (radius > 60) continue;
	        if (contourArea(contour) < 250) continue;
		
		    /*cp.arr[num_elm*3+X] = htonl(center.x);
		    cp.arr[num_elm*3+Y] = htonl(center.y);
		    double z = 0;
		    
		    radius *= 2.5;
		    sem_wait(&semmy);
		    minMaxLoc(depth_filtered(Rect(std::min(std::max(center.x-radius, 0.0f), FREENECT_FRAME_W-2*radius-1), std::min(std::max(center.y-radius, 0.0f), FREENECT_FRAME_H-2*radius-1), radius*2, radius*2)), &z);
		    sem_post(&semmy);
		    
		    printf("z: %f\n", z);
		    z = (d_floor - z) * 1000.0/d_floor;
		    
		    if (z < 0) continue;*/
		    
		    //circle(frame, center, radius/2.5, CV_RGB(255, 0, 0), 3);
		    
		    //vertices.push_back(Vertex(center.x, center.y, z, LINE_RED, LINE_GREEN, LINE_BLUE));
		    
		    /*int c = iter->size();
		    const Point* points = &((*iter)[0]);
		    polylines(frame, &points, &c, 1, true, CV_RGB(255, 0, 0), 3);*/
		    drawContours(frame, contours, i, CV_RGB(255, 0, 0), 3);
		    
            /*cp.arr[num_elm*3+Z] = htonl((int)z);
            printf("Found bright spot\tx: %.0f\ty: %.0f\tz: %.0f\n", center.x, center.y, z);
		    num_elm++;*/
	    }
	    cp.size = htonl(num_elm);
	    
	    if (num_elm > 0 && isDrawing)
	    {
	        if (!detected) cp.seq = htonl(ntohl(cp.seq)+1);
	        
            printf("cp.size = %d\tcp.seq = %d\tcp.controllid= %d\n", ntohl(cp.size), ntohl(cp.seq), 
	            ntohl(cp.controllerId));
            if(sio_write(serv_fd, (void*)&cp, sizeof(int)*(3+num_elm*3)) < sizeof(int)*(3+num_elm*3))
	            printf("problem sending\n");
        }
    }
    detected = num_elm > 0;
    
    glutPostRedisplay();
	
    imshow("KinectR", threshR);
    imshow("KinectG", threshG);
    imshow("KinectB", thresh);
    //imshow("Depth", colored);
    imshow("Kinect", frame);
    waitKey(10);
}

void depth_callback(freenect_device *dev, void *depth, uint32_t timestamp){
    Mat frame(FREENECT_FRAME_H, FREENECT_FRAME_W, CV_16UC1, depth);
    
    sem_wait(&semmy);
    Mat blurred, cvtd;
    frame /= 8;
    frame.convertTo(cvtd, CV_8UC1);
    GaussianBlur(cvtd, blurred, Size(5,5), 3);
    if (depth_init)
    {
        if (depth_init==DEPTH_INIT_COUNT)
            base_depth = blurred;
        else
            base_depth = min(base_depth, blurred);
        depth_init--;
    }
    depth_diff = base_depth - blurred;
    depth_filtered = blurred;
    sem_post(&semmy);
    
    imshow("Depth", depth_diff);
    waitKey(10);
}

void * do_work(void *nil){
    pthread_detach(pthread_self());
    
    int x = 0;
	while (freenect_process_events(ctx)>=0)
	{
		usleep(500);
		if (!(x++ % 200))
		{
		    serialport_read(sfd, &isDrawing);
    		printf("%d\n", isDrawing);
    	}
    }
    return NULL;
}

int main(int argc, char* argv[]){
	sio_t conn;
	
	if (argc < 2)
	{
	    printf("Usage: kinect <depth floor>\n");
	    return 0;
	}
	
	d_floor = atoi(argv[1]);
	
	printf("Starting up . . . da shield!\n");
	if(knct_init() < 0)
		return EXIT_FAILURE;
	sleep(2);
	printf("About to connect to the server\n");
	if((serv_fd = open_host_fd("127.0.0.1", PORT)) < 0){
		printf("Can't connect to the server!!\n"); 
		/*return EXIT_FAILURE;*/
	}
	
	const char* sdev = "/dev/ttyUSB0";
	sfd = serialport_init(sdev, 57600);
	if(sfd == 0){
		printf("Failed to init serial\n");
		return -1;
	}
	
	namedWindow("Kinect");
	namedWindow("KinectR");
	namedWindow("KinectG");
	namedWindow("KinectB");
	namedWindow("Depth");
	
	cp.controllerId = htonl(1);
	cp.seq = htonl(-1);
	
	glutInit(&argc, argv);
	glutInitWindowSize(512, 512);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutCreateWindow("Line display");

	if(0)
		glutFullScreen();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glLoadMatrixf(mproj);
	//glOrtho(0.0, 10.0, 0.0, 10.0, -10.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glLoadMatrixf(mmv);
	
	tbInit(GLUT_LEFT_BUTTON);
    tbAnimate(GL_TRUE);
	
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMouseFunc(tbMouse);
    glutMotionFunc(tbMotion);
    glutIdleFunc(glutPostRedisplay);
    atexit(knct_cleanup);
	
	printf("Connected to server\n");
	sio_init_read(&conn, serv_fd);
	
	
	unsigned char* vidcb_buf = (unsigned char*)malloc(FREENECT_VIDEO_RGB_SIZE*sizeof(unsigned char));
	uint16_t* depthcb_buf = (uint16_t*)malloc(FREENECT_DEPTH_11BIT_SIZE);
	sem_init(&semmy, 0, 1);
	freenect_set_video_buffer(dev, vidcb_buf);
	freenect_set_video_format(dev, FREENECT_VIDEO_RGB);
	freenect_set_depth_buffer(dev, depthcb_buf);
	freenect_set_depth_format(dev, FREENECT_DEPTH_11BIT);
	freenect_set_led(dev, LED_BLINK_RED_YELLOW);
	freenect_set_video_callback(dev, vid_callback);
	freenect_set_depth_callback(dev, depth_callback);
	printf("About to start video\n");
	freenect_start_video(dev);
	printf("Video started\nAbout to start depth\n");
	if(freenect_start_depth(dev) < 0)
		printf("Error starting depth\n");
	printf("Depth started\n");
	
	pthread_t worker;
	pthread_create(&worker, NULL, do_work, NULL);
	
	glutMainLoop();
}

