//
//SampleClient.java
//
//Main client class. Can be both an Android or Kinect client. 
//At this moment, readBytes() method is more important.  
//But no tester code written for sendBytes() though method worked in previous round of testing.
//
//Created by Yumin Wong on 2/25/11.
//Last modified by Yumin Wong on 4/22/11.
//

//package comm_protocol;

import java.io.*;
import java.net.*;
import java.util.*;

//Client side
public class SampleClient
{
	private Socket socket;
	private String hostname;
	private int port;
	
	private final LinkedList<CoordPacket.Coord> coordQueue;
	
	/**
	 * Constuctor for client that will connect to <hostname> at <port>
	 */
	public SampleClient(String hostname, int port)
	{
		this.hostname = hostname;
		this.port = port;
		coordQueue = new LinkedList<CoordPacket.Coord>();
	}
	
	/**
	 * Client establishes connection with <hostname> at <port> and creates 
	 * a new socket there.
	 * @return true if connection was successfully established.
	 */
	public boolean connect()
	{
		try
		{
			socket = new Socket(hostname, port);
			return true;
		}
		catch(Exception e)
		{
			System.out.println("Exception caught in connect().");
			e.printStackTrace();
			return false;
		}
	}
	
	/**
	 * Close socket. Connection terminated.
	 */
	public void close()
	{
		if (socket != null)
		{
			try
			{
				socket.close();
			} 
			catch (Exception e)
			{
				System.out.println("Exception caught in close().");
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Returns true when CoordPacket is "stripped" and sent as byte[] array successfully.
	 * @param coordPack The coordinate packet that contains the data to send.
	 * @return true if method succeeds, false if there is an exception.
	 */
	public boolean sendBytes(CoordPacket coordPack)
	{
		try
		{
			coordPack.toByteStream(new DataOutputStream(socket.getOutputStream()));
			return true;
		}
		catch (Exception e)
		{
			//System.out.println("Exception caught in writeMessage()");
			return false;
		}
	}
	
	/**
	 * Reads bytes of data from socket's input stream -- data is coming from server.
	 * @return true if data is successfully read.
	 */
	public boolean readBytes()
	{
		try
		{
			CoordPacket cp = CoordPacket.fromByteStream(new DataInputStream(socket.getInputStream()));
			
			//coordQueue.offer(cp);
			System.out.println(cp.toString());
			
			return true;
		}
		catch (Exception e)
		{
			//e.printStackTrace();
			return false;
		}
	}
	
	
	public static void main (String[] args)
	{
		final SampleClient client = new SampleClient("127.0.0.1", 1096);
		client.connect();
		
		if(args.length > 0 && args[0].equals("kinect"))
		{
			System.out.println("In SampleClient main(), client sending bytes.");
			
			Scanner in = new Scanner(System.in);
			
			while(!client.socket.isClosed())
			{
			    int[] points = new int[3];
				System.out.print("Seq: ");
				int seq = in.nextInt();
				System.out.print("Coords: ");
				points[0] = (int)in.nextFloat();
				points[1] = (int)in.nextFloat();
				points[2] = (int)in.nextFloat();
				
				CoordPacket cp = new CoordPacket(1);
				cp.setControllerId(1);
				cp.setSeq(seq);
				cp.setPoint(0, points);
				
			    System.out.println(cp.toString());
				client.sendBytes(cp);
			}
		}
		else
		{
			System.out.println("In SampleClient main(), client reading bytes.");
			// Just for testing purposes.
			while(!client.socket.isClosed())
			{
				client.readBytes();
			}
		}
		
		client.close();
	}
}
